<?php
/*
* 2022 smartbill.ro
*
*  @author smartbill.ro <contact@smartbill.ro>
*  @copyright  2022 smartbill.ro
*/
define( 'SMRT_PLUGIN_VERSION', '3.1.9' );
define( 'SMRT_DATABASE_INVOICE_STATUS_DRAFT', 0 );
define( 'SMRT_DATABASE_INVOICE_STATUS_FINAL', 1 );
define( 'SMRT_DOCUMENT_TYPE_INVOICE', 0 );
define( 'SMRT_DOCUMENT_TYPE_ESTIMATE', 1 );

require_once __DIR__ . '/classes/SmartbillSettings.php';
require_once __DIR__ . '/classes/smartbill-cloud-rest-client.php';
require_once __DIR__ . '/classes/SmartbillUtils.php';
require_once __DIR__ . '/classes/SmartbillDataLogger.php';
require_once __DIR__ . '/classes/SmartbillProduct.php';

use PrestaShop\Module\AutoUpgrade\AjaxResponse;
use PrestaShop\Module\AutoUpgrade\BackupFinder;
use PrestaShop\Module\AutoUpgrade\Parameters\UpgradeFileNames;
use PrestaShop\Module\AutoUpgrade\UpgradePage;
use PrestaShop\Module\AutoUpgrade\UpgradeSelfCheck;
use PrestaShop\Module\AutoUpgrade\Tools14;
use PrestaShop\Module\AutoUpgrade\UpgradeContainer;
use PrestaShop\Module\AutoUpgrade\UpgradeTools\FilesystemAdapter;
class SmartbillTab extends AdminController {

	public function __construct() {
		 parent::__construct();
		if ( ! isset( $_GET['action'] ) && empty( $_GET['action'] ) ) {
			Tools::redirectAdmin(
				$this->context->link->getAdminLink( 'AdminModules' ) .
				'&module_name=smartbill&configure=smartbill'
			);
		}
	}

	public function ajaxProcessExportDocument() {
		$options = SmartBillSettings::get_config_fields_values();
		$utils   = new SmartBillUtils();
		try {
			$order_id         = (int) Tools::getValue( 'id_order' );
			$smartbillInvoice = $utils->generateInvoiceData( $order_id );

			$client = new SmartBill_Cloud_REST_Client( $options['SMARTBILL_EMAIL'], $options['SMARTBILL_API_TOKEN'] );
			$client->setPrestashopOrderId( $order_id );
			$invoiceLogger = new SmartBill_Data_Logger( $order_id );
			$client->setDataLogger( $invoiceLogger );

			if ( (int) $options['SMARTBILL_DOCUMENT_TYPE'] === SMRT_DOCUMENT_TYPE_INVOICE ) {
				$serverCall = $client->createInvoiceWithDocumentAddress( $smartbillInvoice );
			} elseif ( (int) $options['SMARTBILL_DOCUMENT_TYPE'] === SMRT_DOCUMENT_TYPE_ESTIMATE ) {
				$serverCall = $client->createProformaWithDocumentAddress( $smartbillInvoice );
			} else {
				throw new \Exception( Context::getContext()->getTranslator()->trans( 'Tipul de document emis este invalid.', array(), 'Modules.Smartbill.Admin' ) );
			}

			if ( $serverCall['errorText'] ) {
				$return['status']  = false;
				$return['message'] = $serverCall['message'];
				$return['error']   = $serverCall['errorText'];
				$return['orderId'] = $order_id;
			} else {
				$return['status'] = true;
				if ( isset( $serverCall['number'] ) && ( $serverCall['number'] ) ) {
					$return['message'] = Context::getContext()->getTranslator()->trans( 'Documentul a fost emis cu succes: ', array(), 'Modules.Smartbill.Admin' ) . $serverCall['message'] . $serverCall['series'] . ' ' . $serverCall['number'] . '.';
					$invoiceLogger->set_data( $order_id, 'smartbill_invoice_id', $serverCall['number'] )
					->set_data( $order_id, 'smartbill_series', $serverCall['series'] )
					->set_data( $order_id, 'smartbill_headers', $serverCall['get_headers'] )
					->set_data( $order_id, 'smartbill_document_url', $serverCall['documentUrl'] )
					->set_data( $order_id, 'smartbill_status', SMRT_DATABASE_INVOICE_STATUS_FINAL )
					->save( $order_id );

				} else {
					if ( isset( $serverCall['series'] ) ) {
						$invoiceLogger->set_data( $order_id, 'smartbill_series', $serverCall['series'] );
					}
					$invoiceLogger
					->set_data( $order_id, 'smartbill_document_url', $serverCall['documentUrl'] )
					->set_data( $order_id, 'smartbill_headers', $serverCall['get_headers'] )
					->set_data( $order_id, 'smartbill_status', SMRT_DATABASE_INVOICE_STATUS_DRAFT )
					->save( $order_id );

					$return['message'] = Context::getContext()->getTranslator()->trans( 'Operatiunea s-a desfasurat cu succes: ', array(), 'Modules.Smartbill.Admin' ) . $serverCall['message'];
				}
				$return['number']  = $serverCall['number'];
				$return['series']  = $serverCall['series'];
				$return['status']  = true;
				$return['headers'] = $serverCall['get_headers'];
				// Return json result
				$return['documentViewUrl'] = $serverCall['documentViewUrl'];
				$return['documentUrl']     = str_replace( '/editare/', '/vizualizare/', $serverCall['documentUrl'] );
			}
			// Update orders table field
			Db::getInstance()->execute( 'UPDATE  ' . _DB_PREFIX_ . "orders SET `smartbill_document` = '{$return['documentUrl']}' WHERE id_order = {$order_id}" );
			Db::getInstance()->execute( 'UPDATE  ' . _DB_PREFIX_ . "orders SET `smartbill_public_invoice` = '{$return['documentViewUrl']}' WHERE id_order = {$order_id}" );
			die( json_encode( $return ) );

		} catch ( Exception $e ) {
			$return['error']   = $e->getMessage();
			$return['message'] = $e->getMessage();
			$return['status']  = false;
			$return['orderId'] = $order_id;
			die( json_encode( $return ) );
		}
	}

	/**
	 * ajaxProcessSendMailDocument
	 * * Ajax hook for sending email with smartbill document to client
	 *
	 * @return JSON encoded array
	 */
	public function ajaxProcessSendMailDocument() {
		$order_id = (int) Tools::getValue( 'id_order' );
		$return   = array();
		if ( ! is_numeric( $order_id ) || ( $order_id <= 0 ) ) {
			$return['status']  = false;
			$return['message'] =  Context::getContext()->getTranslator()->trans( 'Comanda furnizata este invalida', array(), 'Modules.Smartbill.Admin' );
		} else {
			try {
				$order         = new Order( $order_id );
				$options       = SmartBillSettings::get_config_fields_values();
				$utils         = new SmartBillUtils();
				$client_data   = json_decode( json_encode( $utils->_getOrderClientData( $order, $options ) ), true );
				$invoiceLogger = new SmartBill_Data_Logger( $order_id );
				if ( is_null( $invoiceLogger ) ) {
					 throw new Exception( Context::getContext()->getTranslator()->trans( 'Comanada furnizata nu are factura', array(), 'Modules.Smartbill.Admin' ) );
				}
				if ( empty( $options['SMARTBILL_EMAIL'] ) || empty( $options['SMARTBILL_API_TOKEN'] ) ) {
					throw new Exception(  Context::getContext()->getTranslator()->trans( 'Este necesar sa furnizati un utilizator si o parola valide.',array(), 'Modules.Smartbill.Admin' ) );
				}

				$document_series = $options['SMARTBILL_INVOICE_SERIES'];
				$doc_type        = 'factura';
				if ( (int) $options['SMARTBILL_DOCUMENT_TYPE'] === SMRT_DOCUMENT_TYPE_ESTIMATE ) {
					$document_series = $options['SMARTBILL_ESTIMATE_SERIES'];
					$doc_type        = 'proforma';
				}

				$smartbill_email = array(
					'companyVatCode' => $options['SMARTBILL_CIF'],
					'seriesName'     => $document_series,
					'type'           => $doc_type,
					'number'         => $invoiceLogger->get_document_id(),
					'to'             => $client_data['email'],
				);

				if ( $options['SMARTBILL_SEND_MAIL_WITH_DOCUMENT'] == '1' ) {
					$smartbill_email['cc']  = $options['SMARTBILL_SEND_MAIL_CC'];
					$smartbill_email['bcc'] = $options['SMARTBILL_SEND_MAIL_BCC'];
				} else {
					$options['SMARTBILL_SEND_MAIL_CC']  = '';
					$options['SMARTBILL_SEND_MAIL_BCC'] = '';
				}

				if ( ! empty( $options['SMARTBILL_SEND_MAIL_CC'] ) ) {
					$m_bcc = ', ' . $options['SMARTBILL_SEND_MAIL_BCC'];}
				if ( ! empty( $options['SMARTBILL_SEND_MAIL_BCC'] ) ) {
					$m_cc = ', ' . $options['SMARTBILL_SEND_MAIL_CC'];}

				$message = sprintf( Context::getContext()->getTranslator()->trans( '%4$s a fost trimisa cu succes catre: %1$s%2$s%3$s.', array(), 'Modules.Smartbill.Admin' ), $client_data['email'], $m_cc, $m_bcc, ucwords( $doc_type ) );

				$client     = new SmartBill_Cloud_REST_Client( $options['SMARTBILL_EMAIL'], $options['SMARTBILL_API_TOKEN'] );
				$serverCall = $client->sendDocument( $smartbill_email );

				$return['message'] = $message;
				$return['status']  = true;
				$return['headers'] = $serverCall['get_headers'];

			} catch ( Exception $e ) {
				$return['error']   = $e->getMessage();
				$return['message'] = $e->getMessage();
				$return['status']  = false;
			}
		}

		die( json_encode( $return ) );
	}


	/**
	 * ajaxProcessDownloadStockHistory
	 * * Ajax hook downloading sync stock history
	 *
	 * @return JSON encoded array
	 */
	public function ajaxProcessDownloadStockHistory() {
		$return = array();
		try {
			$file = __DIR__ . DIRECTORY_SEPARATOR . 'classes' . DIRECTORY_SEPARATOR . 'smartbill_sincronizare_stocuri.log';
			if ( ! file_exists( $file ) ) {
				throw new \Exception(  Context::getContext()->getTranslator()->trans( 'Fisier inexistent!', array(), 'Modules.Smartbill.Admin' ) );
			}

			$zip_name = 'smartbill_sincronizare_stocuri.zip';
			$zip      = new ZipArchive();

			$zip_path = __DIR__ . DIRECTORY_SEPARATOR . $zip_name;
			$zip->open( $zip_path, ZipArchive::CREATE | ZipArchive::OVERWRITE );
			$zip->addFile( $file, 'smartbill_sincronizare_stocuri.log' );
			$zip->close();

			header( 'Content-Type: application/octet-stream' );
			header( 'Content-Disposition: attachment; filename=' . basename( $zip_name ) );
			header( 'Expires: 0' );
			header( 'Cache-Control: must-revalidate' );
			header( 'Pragma: public' );
			header( 'Content-Length: ' . filesize( $zip_path ) );
			readfile( $zip_path );
			exit;

		} catch ( Exception $e ) {
			$return['error'] = $e->getMessage();
			die( json_encode( $return ) );
		}
	}

	public function ajaxProcessSyncStock() {
		$warehouse = Tools::getValue( 'warehouse' );
		$return = array();

		try {
			
			if ( empty( $warehouse ) ) {
				throw new Exception( Context::getContext()->getTranslator()->trans( 'Lipsa gestiune', array(), 'Modules.Smartbill.Admin' ) );
			}
			$now = new DateTime('now');
			$now->setTimezone(new DateTimeZone(Configuration::get('PS_TIMEZONE')));
			$last_date = Configuration::get( 'SMARTBILL_LAST_SYNC' ); 

			if(!empty($last_date)){
				$last_date = unserialize($last_date);
				if( 5 > (int)$now->diff($last_date)->i ){
					$temp_date = $last_date;
					$temp_date->modify("+5 minutes");
					
					throw new Exception( sprintf(  Context::getContext()->getTranslator()->trans( 'Preluarea manuala a stocurilor se poate face doar o data la 5 minute. Urmatoarea preluare va fi posibila la ora %s.', array(), 'Modules.Smartbill.Admin' ),$temp_date->format('H:i') ) );
				}
			}

			$options = SmartBillSettings::get_config_fields_values();
			if ( empty( $options['SMARTBILL_EMAIL'] ) || empty( $options['SMARTBILL_API_TOKEN'] ) ) {
				throw new Exception( Context::getContext()->getTranslator()->trans( 'Este necesar sa furnizati un utilizator si o parola valide.', array(), 'Modules.Smartbill.Admin' ) );
			}
			
			$client = new SmartBill_Cloud_REST_Client( $options['SMARTBILL_EMAIL'], $options['SMARTBILL_API_TOKEN'] );
			
			if(empty($last_date) || 4 < (int)$now->diff($last_date)->i){
				$stocks = $client->getWareStock( $options['SMARTBILL_CIF'], $warehouse);
			}
			
			
			if(empty($stocks)){
				throw new Exception( Context::getContext()->getTranslator()->trans( 'Eroare sintaxa. Verifica valididatea JSON-ului primit.', array(), 'Modules.Smartbill.Admin' ) );
			}

			if(isset( $stocks['errorText'] ) && !empty( $stocks['errorText'] )){
				throw new Exception( $stocks['errorText'] );
			}
			
			if ( isset( $stocks['products'] ) && 0 < count($stocks['products']) ) {
				$updated_products=0;
				$file = fopen(__DIR__ . DIRECTORY_SEPARATOR . 'Istoric sincronizare manuala SmartBill.csv', 'w');
				fputcsv($file, [ 
						Context::getContext()->getTranslator()->trans( 'Cod produs SmartBill' , array(), 'Modules.Smartbill.Admin' ),
						Context::getContext()->getTranslator()->trans( 'Denumire produs SmartBill' , array(), 'Modules.Smartbill.Admin' ),
						Context::getContext()->getTranslator()->trans( 'Denumire produs Prestashop' , array(), 'Modules.Smartbill.Admin' ),
						Context::getContext()->getTranslator()->trans( 'ID produs Prestashop' , array(), 'Modules.Smartbill.Admin' ),
						Context::getContext()->getTranslator()->trans( 'Stoc nou' , array(), 'Modules.Smartbill.Admin' )
				]);
				
				Configuration::updateValue( 'SMARTBILL_LAST_SYNC', serialize( $now ) );

				foreach($stocks['products'] as $product){
					$product = array_values($product);
					$product = new SmartbillProduct(...$product);
					if( true === $product->sync_quantity($options['SMARTBILL_PRODUCT_SKU_TYPE']) ){
						$date = new DateTime('now');
						$date->setTimezone(new DateTimeZone(Configuration::get('PS_TIMEZONE')));
						Configuration::updateValue( 'SMARTBILL_STOCK_UPDATE', serialize( $date ) );
						$updated_products++;
					}
					fputcsv($file, $product->to_arr());
				}

				$product=null;
				fclose($file);
			}else{
				$error_message = sprintf( Context::getContext()->getTranslator()->trans( 'Nu exista produse pe Gestiunea %s. Acceseaza SmartBill Cloud, adauga produse in gestiune apoi incearca din nou preluarea stocurilor.', array(), 'Modules.Smartbill.Admin' ),$warehouse);
				throw new Exception($error_message);
			}
			
			Configuration::updateValue( 'SMARTBILL_LAST_SYNC', serialize( $now ) );

			if( count($stocks['products']) == $updated_products ){
				$return['icon'] = 'success';
				$return['data'] = sprintf( Context::getContext()->getTranslator()->trans('A fost actualizat stocul pentru %s produs.', array(), 'Modules.Smartbill.Admin'), $updated_products);
				if(1 < $updated_products){
					$return['data'] = sprintf( Context::getContext()->getTranslator()->trans('A fost actualizat stocul pentru %s produse.', array(), 'Modules.Smartbill.Admin'), $updated_products);
				}
			}else{
				$return['icon'] = 'warning';
				$return['data'] = sprintf( Context::getContext()->getTranslator()->trans('A fost actualizat stocul pentru %s produs. Descarca istoricul pentru consultarea erorilor.',  array(), 'Modules.Smartbill.Admin'), $updated_products);

				if( 1 < $updated_products ){
					$return['data'] = sprintf( Context::getContext()->getTranslator()->trans('A fost actualizat stocul pentru %s produse.  Descarca istoricul pentru consultarea erorilor.',  array(), 'Modules.Smartbill.Admin'), $updated_products);
				}
				if( 0 == $updated_products ){
					$return['data'] = sprintf( Context::getContext()->getTranslator()->trans('Nu au fost actualizate stocurile. Descarca istoricul pentru consultarea erorilor.',  array(), 'Modules.Smartbill.Admin'), $updated_products);
					$return['icon'] = 'error';
				}
			}

			die( json_encode( $return ) );

		}catch ( Exception $e ) {
			$return['error'] = $e->getMessage();
			die( json_encode( $return ) );
		}
	}


	public function ajaxProcessDownloadManualStockHistory(){
		$file=__DIR__ . DIRECTORY_SEPARATOR . 'Istoric sincronizare manuala SmartBill.csv';
		$return = array();
		try {
			$file = __DIR__ . DIRECTORY_SEPARATOR . 'Istoric sincronizare manuala SmartBill.csv';
			if ( ! file_exists( $file ) ) {
				throw new \Exception(  Context::getContext()->getTranslator()->trans( 'Fisier inexistent!', array(), 'Modules.Smartbill.Admin' ) );
			}

			$zip_name = 'Istoric sincronizare manuala SmartBill.zip';
			$zip      = new ZipArchive();

			$zip_path = __DIR__ . DIRECTORY_SEPARATOR . $zip_name;
			$zip->open( $zip_path, ZipArchive::CREATE | ZipArchive::OVERWRITE );
			$zip->addFile( $file, 'Istoric sincronizare manuala SmartBill.csv' );
			$zip->close();

			header( 'Content-Type: application/octet-stream' );
			header( 'Content-Disposition: attachment; filename=' . basename( $zip_name ) );
			header( 'Expires: 0' );
			header( 'Cache-Control: must-revalidate' );
			header( 'Pragma: public' );
			header( 'Content-Length: ' . filesize( $zip_path ) );
			readfile( $zip_path );
			exit;
			
		}catch(Exception $e){
			$return['error'] = $e->getMessage();
			http_response_code(404);
			die( json_encode( $return ) );
		}
	}

}
